import os.path
import shutil



class FS_Generator():
    """
    Generate directory structure

    >>> dirStruct = [
    ... {'type': 'dir', 'name': 'test',
    ...    'items': [
    ...        {'type': 'file', 'name': 'test.txt', 'content': "sample content"},
    ...        {'type': 'dir', 'name': 'test11',
    ...          'items': [{'type': 'dir', 'name': 'test111'}]
    ...        }
    ...    ]
    ... },
    ... {'type': 'dir', 'name': 'test2'}
    ... ]


    >>> fg = FS_Generator(dirStruct, '/home/sal/workspace/snappy/')
 
    # Test directory creation
    >>> os.path.exists("/home/sal/workspace/snappy/test")
    True
    >>> os.path.exists("/home/sal/workspace/snappy/test/test11")
    True
    >>> os.path.exists("/home/sal/workspace/snappy/test/test11/test111")
    True

    # Test file content
    >>> os.path.exists("/home/sal/workspace/snappy/test/test.txt")
    True
    >>> handle = open("/home/sal/workspace/snappy/test/test.txt")
    >>> data = handle.read()
    >>> handle.close()
    >>> data
    "sample content"
    
    # Clear all
    >>> fg.clean()
    """

    def __init__(self, struct, home):
        self.struct = struct
        self.home = home
        self.generate()


    def generate(self):
        self.__generate_items_recursive(self.struct, self.home)

    def __generate_items_recursive(self, items, path):
        for e in items:
            if ('dir' == e['type'] and not os.path.exists(path + e['name'])):
                os.mkdir(path + e['name'] + '/')
                
            elif ('file' == e['type'] and not os.path.exists(path + e['name'])):
                self.__create_file(path + e['name'], e['content'])
                
            if 'items' in e:
                self.__generate_items_recursive(e['items'], path + e['name'] + '/')


    def __create_file(self, filename, content):
        handle = open(filename, 'w')
        handle.write(content)
        handle.close()

        
    def clean(self):
        for e in self.struct:
            if ('dir' == e['type'] and os.path.exists(self.home + e['name'])):
                shutil.rmtree(self.home + e['name'])






class FS_Scanner():
    """
    Search files in directory and make simple operation other it

    >>> dirStruct = [
    ...  {'type': 'dir', 'name': 'test',
    ...    'items': [
    ...        {'type': 'file', 'name': 'test.txt', 'content': "aaa"},
    ...        {'type': 'dir', 'name': 'test11',
    ...          'items': [
    ...                      {'type': 'dir', 'name': 'test111'}, 
    ...                      {'type': 'file', 'name': 'test11.txt', 'content': "bbb"}
    ...                   ]
    ...        },
    ...        {'type': 'dir', 'name': 'test2', 'items':[{'type': 'file', 'name': 'test2.txt', 'content': "ccc"}]}
    ...     ]
    ...  }
    ... ]
    >>> fg = FS_Generator(dirStruct, '/home/sal/workspace/snappy/')

    
    >>> fs = FS_Scanner('/home/sal/workspace/snappy/test')
    >>> list = fs.files.sort()
    >>> fs.files
    ['/home/sal/workspace/snappy/test/test.txt', '/home/sal/workspace/snappy/test/test11/test11.txt', '/home/sal/workspace/snappy/test/test2/test2.txt']
    >>> fs.concat()
    "aaabbbccc"

    # Clear all
    >>> fg.clean()
    """


    def __init__(self, path):
        self.home = path
        self.files = []
        self.scan()
        pass

    def scan(self):
        for root, subFolders, files in os.walk(self.home):
            for file in files:
                self.files.append(os.path.join(root, file))
        

    def concat(self):
        result = ""
        for filename in self.files:
            handle = open(filename, 'r')
            data = handle.read()
            handle.close()
            result += data
            
        return result